from signalSingularities import * 
from signalCharacteristic import * 
from signalLeg import *
import numpy as np
import scipy.special as sp
import matplotlib.pyplot as plt
from pylab import *
from legendre import *
# Show approximations of different signals in fixed Legendre space
# Signal 1: Legendre signal with  higher n_leg.
# Signal 2: Spline signal with singularities.
# Signal 3: Signal in characteristic basis.

## low order approximation

n_leg = 4
xvals = np.append(np.arange(-1,1,0.001),1)
#Legendre.initialize(n_leg)
# for the Legendre polynomials the approximation has closed for solution
(fvals_leg, coeffs_leg) = SignalLeg.signal( xvals);
coeffs_leg = coeffs_leg[0:n_leg+1]
fvals_leg_rec = Legendre.signal(coeffs_leg, xvals); 

# signal with Singularities
# The coeffs used for reconstructing the singularity function are calculated by 
# estimated <f_sing(x),P_{0-4}> in interval [-1,1]
# the following numerical results are hard coded and calculated in Matlab with f(x) 
# sampled by halton sampling method in the interval [-1,1] 
coeffs_sing = [0.0201, 0.0580,  0.0022,   0.0205,  0.0649] 
fvals_sing_rec = Legendre.signal(coeffs_sing,xvals) 
fvals_sing = SignalSingularities.signal( xvals);

# characteristic signal
# The coeffs used for reconstructing the characteristic function are calculated 
# by estimated <f_chara(x),P_{0-4}> in interval [-1,1]
# the following numerical results are hard coded and calcualted in 
# Matlab with f(x) sampled by halton sampling method in the interval [-1,1] 
coeffs_char=[0.3076, 0.1589, 0.1256,  -0.2015, -0.1192]
fvals_char_rec = Legendre.signal(coeffs_char,xvals) 
fvals_char = SignalCharacteristic.signal( xvals);

## Plot results

execfile('legPreVisualization.py')

plot( xvals, fvals_leg, '-r', xvals, fvals_leg_rec, '--r');
plot( xvals, fvals_sing, '-b', xvals, fvals_sing_rec, '--b');
plot( xvals, fvals_char, '-c', xvals, fvals_char_rec, '--c');

execfile('legPostVisualization.py')

plt.savefig( 'figures/legPlotApproximations_a.eps')
plt.savefig( 'figures/legPlotApproximations_a.pdf')

### higher order approximation

n_leg = 20;

## for the Legendre polynomials the approximation has closed for solution
(fvals_leg, coeffs_leg) = SignalLeg.signal( xvals);
coeffs_leg = coeffs_leg[0:n_leg+1]
fvals_leg_rec = Legendre.signal(coeffs_leg, xvals); 

## signal with Singularities
#coeffs_sing = legApproximateSignal( n_leg, @signalSingularities);
coeffs_sing = [0.0201, 0.0580, 0.0022, 0.0205, 0.0649, 0.0483, -0.3192, 0.0578,
               0.1586, -0.1036, 0.0020, -0.0246, -0.0321, 0.0773, 0.0750, -0.0912,
               -0.0621, 0.0936, 0.0182, -0.0659, -0.0265]
fvals_sing_rec = Legendre.signal(coeffs_sing,xvals) 
fvals_sing = SignalSingularities.signal( xvals);

## characteristic signal
#coeffs_char = legApproximateSignal( n_leg, @signalCharacteristic);
coeffs_char = [0.3076, 0.1589, 0.1256, -0.2015, -0.1192, 0.1777, 0.0618, -0.0502,
               -0.0346, -0.0135, 0.0307, 0.0127, -0.0116, -0.0172, -0.0077, 0.0095,
               0.0127, -0.0024, -0.0105, 0.0036, 0.0103]
fvals_char_rec = Legendre.signal(coeffs_char,xvals) 
fvals_char = SignalCharacteristic.signal( xvals);

### Plot results
plt.figure()

execfile('legPreVisualization.py')

plot( xvals, fvals_leg, '-r', xvals, fvals_leg_rec, '--r');
plot( xvals, fvals_sing, '-b', xvals, fvals_sing_rec, '--b');
plot( xvals, fvals_char, '-c', xvals, fvals_char_rec, '--c');

execfile('legPostVisualization.py')

plt.savefig( 'figures/legPlotApproximations_b.eps')
plt.savefig( 'figures/legPlotApproximations_b.pdf')


