# Biorthogonal basis pair

execfile('legInit.py')

## Construct basis pair

# basis
# coeff_scaling = 0.55; 
# B = coeff_scaling * randn( n_leg+1, n_leg+1)
# assert( cond(B) < Inf);
# Fix B for reproducibility
B = np.array([[-0.2254,   -0.8068,    0.1413,   -0.9313,   -0.2281],
     [-0.2769,   -0.8942,   -0.5358,   -0.2472,    1.0517],
     [0.6783,   -1.0806,   -0.6305,   -0.0464,   -0.2150],
     [0.3357,    1.4329,    0.3012,   -1.0956,    0.2251],
     [0.0325,    0.5348,    0.8608,    0.4627,   -0.6283]]) 

# construct dual basis
Bt = np.linalg.inv(B)

## We hence have: 
## \bar{\phi}_i = B(i,:);
## \tilde{e}_j = Bt(:,j);

### Verify biorthogonality of basis

err_biortho = 0.0

for i in range (0, n_leg+1):
  for j in range (0, n_leg+1):
    # compute inner product of primary and dual basis functions:
    # < phi_i , phi_j >
    err = np.dot(B[i,:], Bt[:,j])
    
    # diagonal terms have to be 1.0
    if i == j:
      err = err - 1.0
    
    # pdate total error
    err_biortho = err_biortho + err

## Obviously, the biorthogonality satisfied by construction since 
## Bt = inv(B)
print('Error in biorthogonality = '+str(err_biortho))

### Verify basis property through reconstruction of arbitrary signal

v = rand(n_leg+1)

## compute basis function coefficients using dual basis functions
v_i = np.zeros(n_leg+1)
for i in range (0 , n_leg+1 ):
  v_i[i] = np.dot(v, Bt[:,i])

## reconstruction
v_rec = zeros(n_leg+1)
for i in range( 0 , n_leg+1): 
  v_rec = v_rec + v_i[i] * B[i,:]


## reconstruction error 
err_rec = np.linalg.norm( v - v_rec)

## display result
print('Reconstruction error for random signal (primary random basis): '\
      +str(err_rec))

## demonstrate that basis and dual basis are interchangeable

## compute basis function coefficients using dual basis functions
v_i = np.zeros(n_leg+1)
for i in range (0 , n_leg+1 ):
  v_i[i] = np.dot(v, B[i,:])

## reconstruction
v_rec = zeros(n_leg+1)
for i in range( 0 , n_leg+1): 
  v_rec = v_rec + v_i[i] * Bt[:,i]

## accumulated reconstruction error
err_rec = np.linalg.norm( v - v_rec)

## display result
print('Reconstruction error for random signal (dual random basis): '\
      +str(err_rec))

### Display

## evaluate signals for plotting
xvals = arange(-1.0,1.0, 0.001)



## basis functions
y_bf = np.zeros((len(B),len(xvals)))
for n in range(0,len(B)):
  y_bf[n,:] = Legendre.signal(B[n,:], xvals) 
## dual basis functions
y_dbf = np.zeros((len(B),len(xvals)))
for n in range(0,len(B)):
  y_dbf[n,:] = Legendre.signal(Bt[:,n], xvals)


###
## display basis functions
execfile('legPreVisualization.py')
for n in range(0 , n_leg+1):
  plot(xvals,y_bf[n,:],'-',color=cols[n+1])

execfile('legPostVisualization.py')
plt.savefig( 'figures/legPlotBiorthoBasis_a.eps')
plt.savefig( 'figures/legPlotBiorthoBasis_a.pdf')


###
## display dual basis functions
plt.figure()
execfile('legPreVisualization.py')
for n in range(0 , n_leg+1):
  plot(xvals,y_dbf[n,:],'--',color=cols[n+1])

execfile('legPostVisualization.py')
plt.savefig( 'figures/legPlotBiorthoBasis_b.eps')
plt.savefig( 'figures/legPlotBiorthoBasis_.pdf')
