# Plot Gauss-Legendre quadrature

execfile("legInit.py");

# zero crossings of P_5 (computed with Mathematica)
lambdas = [ -0.90618, -0.538469, 0.0, 0.538469, 0.90618];

# generate values
K = zeros( (n_leg + 1, n_leg+1));

# construct kernel matrix: 
# - each column is fixed basis function
# - each row is one fixed lambda
for n in range( 0 , n_leg+1):
  coeffs = Legendre.eval( n, lambdas);
  K[:,n] = coeffs;  


# sampling matrix
# - i-th column is formed by basis function coefficients for i-th dual
#   basis function 
S = inv(K);

# Legendre polynomials are a special case since all but the first basis
# function vanishes
# The constant is the integral of the normalized P_0 function
# Note that to compute the quadrature weights we would not have to compute
# the full inverse since only the first row is relevant 
weights = sqrt(2.0) * S[0,:];

# values of kernel and dual kernel functions
vals_k = zeros( (n_leg+1, len(xvals)));
vals_kd = zeros( (n_leg+1, len(xvals)));

for i in range(0, len(lambdas)):
  vals_k[i,:] = Legendre.signal(K[i,:], xvals);
  vals_kd[i,:] = Legendre.signal( S[:,i], xvals);

## visualize

execfile("legPreVisualization.py");
# plot corresponding reproducing points
for i in range (0 , len(lambdas)):
  
  col = cols[i];
  col= [(col[j] == 0.0) *0.8 + col[j] for j in range(0,len(col))]
  plot( lambdas[i], 0.0, 'x', markersize=10, color=col);
  plot( xvals, vals_kd[i,:], '-', color= col);  
  plot( [lambdas[i], lambdas[i]], [0.0, 3.0*weights[i]], '-o', \
        color=cols[i], linewidth=1.25);


execfile("legPostVisualizationTight.py");
ylim( [min( 0.0, 4.0 * min(weights)), max( 3.0, 4.0 * max(weights))]);

plt.savefig( 'figures/legPlotGaussLegendre.eps');
plt.savefig( 'figures/legPlotGaussLegendre.pdf');

