import numpy as np
import scipy.special as sp
import matplotlib.pyplot as plt
from pylab import *
from legendre import *
## Plot integral equation with Gaussian kernel 
#
# g(y) = \int_{-1}^1 P_4(x) * 15 * exp( -(x - y) * 10) dx
#

#execfile('legInit.py');
n_leg = 10
xvals = np.append(np.arange(-1,1,0.001),1)
Legendre.initialize(n_leg)
# evaluate input signals
P_4 =  Legendre.eval( 4, xvals);
kernel = exp( -(np.multiply(xvals, xvals) / (1.0 / 10.0)))

y = xvals;
# Closed form solution for integral equation
# (obtained with Mathematica, see legIntegralEquation.nb

erf1=np.array([math.erf(x) for x in sqrt(10.0) * (1.0 - y)])
erf2=np.array([math.erf(x) for x in sqrt(10.0) * (1.0 + y)])
g = (3.0 / (sqrt(2) * 2580.0)) * exp(-10.0 * np.power((1.0 + y),2)) * (20.0 * (-41.0 + 5.0 * y * (11.0 + 28.0 * (-1.0 + y) * y) - exp(40.0 * y) * (41.0 + 5.0 * y * (11.0 + 28.0 * y * (1.0 + y)))) + exp(10.0 * np.power((1.0 + y),2)) * sqrt( 10.0 * pi ) * (141.0 + 40.0 * np.power(y,2) * (-39.0 + 70.0 * np.power(y,2))) * (erf1 + erf2));


## Plotting

execfile('legPreVisualization.py');

## plot inputs
plot( xvals, P_4, '-b');
plot( xvals, kernel, '-g');
   
## plot result
plot( xvals, g, '-r');

## set parameters for saving
execfile('legPostVisualization.py');

## save image
plt.savefig( 'figures/legPlotIntegralEquation.eps')
plt.savefig( 'figures/legPlotIntegralEquation.pdf')

###

## approximation of kernel in Legendre polynomials
K_4 = np.array([[ 7.65749, 0., -1.05591, 0., -0.410261], 
       [0., 6.19499, 0., -1.53588, 0.], 
       [-1.05591, 0., 4.83936, 0., -1.54457], 
       [0., -1.53588, 0., 3.64452, 0.],
       [-0.410261, 0., -1.54457, 0., 2.64278 ]]);
K_4 = K_4 / 3.0;


K_10 = np.array([[ 2.5525, 0., -0.351971, 0., -0.136754, 0., 0.00129759, 0., 0.0109674, 0., -0.000210142], 
        [0., 2.065, 0., -0.511961, 0., -0.104676, 0., 0.0245183, 0., 0.0078762, 0.],
        [-0.351971, 0., 1.61312, 0., -0.514857, 0., -0.0378005, 0., 0.0287731, 0., 0.00140527],
        [0., -0.511961, 0., 1.21484, 0., -0.447322, 0., 0.0107779, 0., 0.0210864, 0.],
        [-0.136754, 0., -0.514857, 0., 0.880925, 0., -0.353749, 0., 0.0346478, 0., 0.0110563],
        [0., -0.104676, 0., -0.447322, 0., 0.614515, 0., -0.260133, 0., 0.0401078, 0.],
        [0.00129759, 0., -0.0378005, 0., -0.353749, 0., 0.412126, 0., -0.179847, 0., 0.0354206],
        [0., 0.0245183, 0., 0.0107779, 0., -0.260133, 0., 0.265628, 0., -0.117666, 0.],
        [0.0109674, 0., 0.0287731, 0., 0.0346478, 0., -0.179847, 0., 0.164514, 0., -0.0731622],
        [0., 0.0078762, 0., 0.0210864, 0., 0.0401078, 0., -0.117666, 0., 0.0979115, 0.],
        [-0.000210142, 0., 0.00140527, 0., 0.0110563, 0., 0.0354206, 0., -0.0731622, 0., 0.056009]]);

## input signal: P_4
coeffs_f_4 = [ 0, 0, 0, 0, 1];
coeffs_f_10 = [ 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0];
## compute approximation to g(x) using finite representation
coeffs_g_a_4 = K_4 * coeffs_f_4;
coeffs_g_a_10 = K_10 * coeffs_f_10;
## reconstruct approximated signal
g_a_4 = Legendre.signal( coeffs_g_a_4[:,4], xvals);
g_a_10 = Legendre.signal( coeffs_g_a_10[:,4], xvals);

plt.figure()
execfile('legPreVisualization.py');

plot( xvals, g, '--r');
plot( xvals, g_a_4, '-b');
plot( xvals, g_a_10, '-g');

## set parameters for saving

execfile('legPostVisualization.py');

## save image
plt.savefig( 'figures/legPlotIntegralEquationApprox.eps')
plt.savefig( 'figures/legPlotIntegralEquationApprox.pdf')



