from legKernelMatrix import *
from legEnKernelMatrix import *

from scipy import optimize

def legOptProgressCallback( xk):
#
#
#

  # collect data only through iterations
  K = legKernelMatrix( n_leg, xk);
  en = cond(K) - 1.0;
    
  ## save current energy 
  history[0]=history[0]+[en]
  history[1]=history[1]+[xk]

  return


# optimize sampling points and plot development of location, also 
# analyze difference in reconstruction error

execfile("legInit.py");

# construct initial sampling points
# lambdas_init = halton( n_leg+1, 1);
# lambdas_init = rand( n_leg+1, 1);
# lambdas_init = sort( (2.0 * lambdas_init) - 1.0)';
# Fix points for reproducibility
lambdas_init = [-0.8866, -0.6438, -0.3283, -0.2807, 0.0438];


# initial condition number
cond_init = cond( legKernelMatrix( n_leg, lambdas_init));

## variable to retain optimization history
history= np.array([[],[lambdas_init]]);
history[0]=[cond_init-1];

## generate lower and upper bounds for locations for optimization
lb = -1.0 * ones( size(lambdas_init));
ub = ones( len(lambdas_init));
bound = map(np.append, lb, ub)
## use active set algorithm 
#options = optimset( 'Algorithm', 'active-set', 
                    #'OutputFcn', @legOptProgressCallback,
                    #'Display', 'iter', 
                    #'TolFun', 1e-12, 
                    #'TolX', 1e-10);

## construct the energy function that only has one parameter
enFunc = legEnKernelMatrix;




#lambdas_opt=optimize.fmin_slsqp(enFunc, lambdas_init, bounds=[(-1,1),(-1,1),(-1,1),(-1,1),(-1,1)], fprime_ieqcons=None, args=[n_leg], iter=100, acc=1.0e-12, iprint=1, disp=True, full_output=1, epsilon=1.0e-10)[0]#,callback=legOptProgressCallback)

## run optimization
lambdas_opt=optimize.minimize(enFunc, 
                              lambdas_init, 
                              args=[n_leg], 
                              method='BFGS',  ## TODO: fix 
                              ## BFGS method chosen so that intermediate 
                              ## results show, but doesn't take bounded constraints
                              bounds=bound, 
                              tol=1e-12, 
                              callback=legOptProgressCallback,
                              options={'eps': 1e-10,'disp':True}).x

                       


### 
cond_opt = cond( legKernelMatrix( n_leg, lambdas_opt));

### Visualization convergence

execfile("legPreVisualization.py");
plot( arange(0,len(history[0])), np.array(history[0]) + 1.0, '-xr');
execfile("legPostVisualization.py");
xlim( [0, len(history[0])-1]);
ylim( [1, max(history[0])+ 1]);

yscale('log')
yticks([1e0, 1e1, 1e2, 1e3])

plt.savefig( 'figures/legPlotOptimization_convergence.eps')
plt.savefig( 'figures/legPlotOptimization_convergence.pdf')

### Visualization point evolution

cols_opt = [[1.0, 0.0, 0.0],
            [0.0, 1.0, 0.0],
            [0.0, 0.0, 1.0],
            [0.0, 0.0, 0.0],
            [0.75, 0.0, 0.75]];
cols_opt_s = [[ 1.0, 0.8, 0.8],
              [0.8, 1.0, 0.8],
              [0.8, 0.8, 1.0],
              [0.8, 0.8, 0.8],
              [0.75, 0.8, 0.75]];

## plot first steps, the remaining optimization is very flat

j = 1;
for i in arange(0,len(history[1]),len(history[1])/3):
  plt.figure()
  lambdas = history[1][i];
  K = legKernelMatrix( n_leg, lambdas);

  execfile("legPreVisualization.py");
  
  # avoid visibility issue by drawing lines first
  for a in range( 0 , len(lambdas)):
    kernel = Legendre.signal( K[a,:], xvals);
    plot( xvals, kernel, '-', color=cols_opt_s[a]);

  for a in range( 0 , len(lambdas)):
    plot( lambdas[a], 0.0, 'x', color=cols_opt[a], markersize= 15, linewidth=1.25);
  
  execfile("legPostVisualization.py");
  
  ylim( [-3.0, 3.0]);
  plt.savefig( 'figures/legPlotOptimization_progress_'+str(j)+'.eps')
  plt.savefig( 'figures/legPlotOptimization_progress_'+str(j)+'.pdf')  

  j = j + 1;
  
  pause(0.5);
  

plt.figure()
lambdas = history[1][len(history[1])-1];
K = legKernelMatrix( n_leg, lambdas);

execfile("legPreVisualization.py");
for a in range( 0 , len(lambdas)):
  kernel = Legendre.signal( K[a,:], xvals);
  plot( xvals, kernel, '-', color=cols_opt_s[a]);

for a in range( 0 , len(lambdas)):
  plot( lambdas[a], 0.0, 'x', color=cols_opt[a], markersize= 15, linewidth=1.25);

  execfile("legPostVisualization.py");  
  ylim( [-3.0, 3.0]);

plt.savefig( 'figures/legPlotOptimization_progress_final.eps')
plt.savefig( 'figures/legPlotOptimization_progress_final.pdf')

 