from legRKApproximate import *
from legKernelMatrix import *

# Demonstration how oversampling an optimization affects the approximation
# power using RKHS.

n_leg = 4
xvals = np.append(np.arange(-1,1,0.001),1)
Legendre.initialize(10)
n_leg_in = 5;

# random input signal
# coeffs_in = 0.5 * randn( 1, n_leg_approx+1);
# Fix coefficients for reproducibility
coeffs_in = [0.5000, -0.8321, -0.2950, -0.1390, 0.2114, -0.8351];

# critically sampled
lambdas_cs = [-0.8000, -0.4000, 0.0000, 0.4000, 0.8000];
# oversampled
lambdas_os = [-0.9416, -0.8290, -0.7652, -0.4750, -0.4066, \
              -0.3624, -0.1517, 0.0157, 0.6020, 0.8577];
# oversampled and optimized
lambdas_opt = [-0.9197, -0.6570, -0.5523, -0.2614, -0.1292, \
               0.1557, 0.2259, 0.5768, 0.6408, 0.9204];

# compute approximations
f_cs = legRKApproximate( n_leg_in, n_leg, lambdas_cs, coeffs_in, xvals);
f_os = legRKApproximate( n_leg_in, n_leg, lambdas_os, coeffs_in, xvals);
f_opt = legRKApproximate( n_leg_in, n_leg, lambdas_opt, coeffs_in, xvals);

end = len(coeffs_in)
## compute errors
err_cs = norm( coeffs_in[0:end-1] - f_cs['coeffs']);
err_os = norm( coeffs_in[0:end-1] - f_os['coeffs']);
err_opt = norm( coeffs_in[0:end-1] - f_opt['coeffs']);

### Visualization

## input signal
f = Legendre.signal( coeffs_in[0:n_leg_in+1], xvals);
## L_2 optimal approximation
f_ideal = Legendre.signal(coeffs_in[0:n_leg+1], xvals);

execfile("legPreVisualization.py");
plot( xvals, f, '--', color =[0.7, 0.7, 0.7]);
plot( xvals, f_cs['vals'], '-', color=[0.5, 0.5, 1.0]);
plot( xvals, f_os['vals'], '-', color=[0.5, 1.0, 0.5]);
plot( xvals, f_ideal, '-k');
plot( xvals, f_opt['vals'], '-r');
execfile("legPostVisualizationTight.py");

plt.savefig( 'figures/legPlotRKApproximation.eps');
plt.savefig( 'figures/legPlotRKApproximation.pdf');


## plot condition numbers and error rates
print( 'cond = %10.2f , %10.2f , %10.2f' %(f_cs['condK'], f_os['condK'],\
                                           f_opt['condK']));
print( 'err =  %10.2f , %10.2f , %10.2f ' %(err_cs, err_os, err_opt));