# Plot reproducing kernels at three points in [-1,1]

import numpy as np
import scipy.special as sp
import matplotlib.pyplot as plt
from pylab import *
from legendre import *
# Setting basic / common parameters

n_leg = 4
xvals = np.append(np.arange(-1,1,0.001),1)
Legendre.initialize(10)

# locations of reproducing point
lam = 0.5;

# generate values
vals_rk = zeros( len(xvals));
coeffs_rk = zeros(  n_leg+1);

for n in range (0 , n_leg+1):
  
  # value of basis functions
  cdat = Legendre.eval( n, xvals);
  # basis function coefficients
  coeffs = Legendre.eval( n, lam);
  coeffs_rk[n] = coeffs;
  
  # accumulate contribution by current basis function
  vals_rk = vals_rk + coeffs * cdat;
  


# generate random signal 
# coeffs_rand = 0.5 * randn( 1, n_leg + 1);
# fix coefficients for reproducibility
coeffs_rand = np.array([0.1162,0.2132,-0.1864,-0.1182,1.0118]);
sig = Legendre.signal( coeffs_rand, xvals);

# evaluate inner product
ip = np.dot( coeffs_rk, coeffs_rand);


# coeffs_rand_2 = 0.5 * randn( 1, 10 + 1);
# Fix coefficients for reproducibility
coeffs_rand_2 = [ 0.7025, 0.5171, 0.1458, -0.3888, 0.2833, -0.6913, 
                  0.1222, 0.4042, 0.1065, 0.4398, 1.0194];
sig_2 = Legendre.signal(coeffs_rand_2, xvals);

ip_2 = np.dot( coeffs_rk, coeffs_rand_2[0:len(coeffs_rk)]);


## visualize

execfile('legPreVisualization.py');

# plot reproducing kernels
plot( xvals, vals_rk, '-b');

# plot random signal
plot( xvals, sig, '-r');
# plot reproducing property
plot( lam, ip, 'xr')#, 'MarkerSize', 15);

# plot random signal
plot( xvals, sig_2, '-c');
# plot reproducing property
plot( lam, ip_2, 'xc')#, 'MarkerSize', 15);

execfile('legPostVisualization.py');
ylim( [-2.5, 2.5])

#savePlot( h_fig, 'legPlotReproducingProperty');
plt.savefig( 'figures/legPlotReproducingProperty.eps')
plt.savefig( 'figures/legPlotReproducingProperty.pdf')