# Quadrature rule for the space 

execfile("legInit.py");

optimized = 1;

if (1 == optimized):
  # generated with legPlotOptimization.m
  lambdas = [-0.8810,  -0.5428,   -0.0000,    0.5428,   0.8810];
else:
  lambdas = [-0.8866, -0.6438, -0.3283, -0.2807, 0.0438];


# generate values
K = zeros(( n_leg + 1, n_leg+1));

# construct kernel matrix: 
# - each column is fixed basis function
# - each row is one fixed lambda
for n in range (0, n_leg+1):
  coeffs = Legendre.eval(n,lambdas);
  K[:,n] = coeffs;  

# sampling matrix
# - i-th column is formed by basis function coefficients for i-th dual
#   basis function 
S = inv(K);

# Legendre polynomials are a special case since all but the first basis
# function vanishes
# The constant is the integral of the normalized P_0 function
# Note that to compute the quadrature weights we would not have to compute
# the full inverse since only the first row is relevant 
weights = sqrt(2.0) * S[0,:];

# values of kernel and dual kernel functions
vals_k = zeros( (n_leg+1, len(xvals)));
vals_kd = zeros( (n_leg+1, len(xvals)));

for i in range(0, len(lambdas)):
  vals_k[i,:] = Legendre.signal(K[i,:], xvals);
  vals_kd[i,:] = Legendre.signal( S[:,i], xvals);

## Verify quadrature for some special cases

err = 0.0;
for i in range(0 , n_leg):
  
  coeffs = zeros( n_leg+1);
  coeffs[i] = 1;
  
  f_vals = Legendre.signal(coeffs, lambdas);
  
  integral = np.dot( f_vals, weights);
  
  # integral vanishes for all but the first Legendre polynomial
  err = integral;
  

err = integral / n_leg;

print( 'Error integrals = '+ str(err));


## visualize

execfile("legPreVisualization.py");
if( 1 == optimized):
  scaling_plot = 1.0;
else:
  scaling_plot = 1.0;


## plot corresponding reproducing points
for i in range (0 , len(lambdas)):
  
  col = cols[i];
  col= [(col[j] == 0.0) *0.8 + col[j] for j in range(0,len(col))]
  plot( lambdas[i], 0.0, 'x', markersize=10, color=col);
  plot( xvals, vals_kd[i,:], '-', color= col);  
  plot( [lambdas[i], lambdas[i]], [0.0, scaling_plot*weights[i]], '-o', \
        color=cols[i], linewidth=1.25);



execfile("legPostVisualizationTight.py");
ylim( [min( 0.0, 1.25 * min(weights)), max( 3.0, 1.25 * max(weights))]);

if( 1 == optimized):
  plt.savefig( 'figures/legPlotQuadratureOptimized_a.eps');
  plt.savefig( 'figures/legPlotQuadratureOptimized_a.pdf');
else:
  plt.savefig( 'figures/legPlotQuadratureOptimized_b.eps');
  plt.savefig( 'figures/legPlotQuadratureOptimized_b.pdf');
  
