# -*- coding: utf-8 -*-
"""
Created on Tue Feb 18 15:12:43 2014

@author: lessig
"""
import numpy as np
import scipy.special as sp
import matplotlib.pyplot as plt
from pylab import *
from legendre import *


class Legendre :
    """Normalized Legendre polynomials."""

    # The class is best described as a namespace that collects the functions
    # needed to work with normalized Legendre polynomials

    # flag if class has been initialized
    __initialized = False
    __n_max = 0
    __leg_norms = np.array( [])
    __legs = []

    @staticmethod
    def __legNorm( n_leg) :
        """Compute norm of n-th Legendre polynomial"""

        # Legendre polynomials are polynomials and can hence be integrated exactly
        # using anti-derivative ad
        # note that norm requires squared function
        ad = np.polyint( sp.legendre( n_leg) * sp.legendre( n_leg));
        leg_n_norm = np.sqrt( ad(1) - ad(-1));

        return leg_n_norm;

    @staticmethod
    def __legNormV2( n_leg) :
        # Calculate Legendre polynomials norm [-1,1] by using the recurrance relation
        leg_n_norm = sqrt(2.0/(2.0*n_leg+1.0)) 
        return leg_n_norm;



    @staticmethod
    def initialize( n_m) :
        """Initialize class

        This function has to be called before any other method can be used.
        """

        Legendre.__n_max = n_m;
        Legendre.__leg_norms = np.zeros( Legendre.__n_max+1);

        for i in range( 0, n_m+1) :
            Legendre.__leg_norms[i] = Legendre.__legNormV2(i);
            Legendre.__legs.append( sp.legendre(i));

        Legendre.__initialized = True;

    @staticmethod
    def eval( n, x) :
        """Evaluate n-th Legendre polynomial."""
        assert( True == Legendre.__initialized);
        assert( n <= Legendre.__n_max);

        return (1.0 / Legendre.__leg_norms[n]) * Legendre.__legs[n](x)

    @staticmethod
    def signal( coeffs, x) :
        """Evaluate signal defined by coeffs in the Legendre basis at x"""

        f = np.zeros( len(x))
        for i in range( 0, len(coeffs)) :
            f += coeffs[i] * Legendre.eval( i, x)

        return  f

